"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameFiBase = void 0;
const external_1 = require("./external");
const ton_connect_sender_1 = require("./ton-connect-sender");
const content_resolver_1 = require("./content-resolver");
class GameFiBase {
    constructor(params) {
        this.walletConnector = params.walletConnector;
        this.assetsSdk = params.assetsSdk;
        if (params.merchant != null) {
            this.merchant = params.merchant;
        }
    }
    get wallet() {
        if (this.walletConnector.wallet == null) {
            throw new Error('Connect a wallet before using it.');
        }
        return this.walletConnector.wallet;
    }
    get walletAccount() {
        return this.wallet.account;
    }
    get walletAddress() {
        return external_1.Address.parseRaw(this.wallet.account.address);
    }
    get merchantAddress() {
        if (this.merchant == null || this.merchant.tonAddress == null) {
            throw new Error('To make payments with TON pass "merchant.tonAddress" parameter to "GameFi.create" method.');
        }
        return this.merchant.tonAddress;
    }
    get merchantJettonAddress() {
        if (this.merchant == null || this.merchant.jettonAddress == null) {
            throw new Error('To make payments with jetton pass "merchant.jettonAddress" parameter to "GameFi.create" method.');
        }
        return this.merchant.jettonAddress;
    }
    buyWithTon(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.transferTon(Object.assign(Object.assign({}, params), { to: this.merchantAddress }));
        });
    }
    transferTon(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.assetsSdk.sender == null) {
                throw new Error('Sender is not configured.');
            }
            return this.assetsSdk.sender.send({
                to: params.to,
                value: params.amount,
                body: params.comment ? this.createMessagePayload(params.comment) : null
            });
        });
    }
    buyWithJetton(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.transferJetton(Object.assign(Object.assign({}, params), { to: this.merchantAddress }));
        });
    }
    transferJetton(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const jetton = this.assetsSdk.openJetton(this.merchantJettonAddress);
            const jettonWallet = yield jetton.getWallet(this.walletAddress);
            const message = {
                amount: params.amount,
                to: params.to,
                responseDestination: this.walletAddress
            };
            if (params.customPayload != null) {
                message.customPayload = this.createMessagePayload(params.customPayload);
            }
            if (params.forwardAmount != null) {
                message.forwardAmount = params.forwardAmount;
            }
            if (params.forwardPayload != null) {
                message.forwardPayload = this.createMessagePayload(params.forwardPayload);
            }
            return jettonWallet.sendTransfer(message);
        });
    }
    openNftCollection(address) {
        return this.assetsSdk.openNftCollection(address);
    }
    openNftSale(address) {
        return this.assetsSdk.openNftSale(address);
    }
    openNftItem(address) {
        return this.assetsSdk.openNftItem(address);
    }
    openNftItemByIndex(collectionAddress, itemIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            const nftAddress = yield this.assetsSdk
                .openNftCollection(collectionAddress)
                .getItemAddress(itemIndex);
            return this.assetsSdk.openNftItem(nftAddress);
        });
    }
    openSbtCollection(address) {
        return this.assetsSdk.openSbtCollection(address);
    }
    openJetton(address) {
        return this.assetsSdk.openJetton(address);
    }
    openJettonWallet(address) {
        return this.assetsSdk.openJettonWallet(address);
    }
    connectWallet(app) {
        return this.walletConnector.openSingleWalletModal(app);
    }
    onWalletChange(...params) {
        const unsubscribe = this.walletConnector.onStatusChange(...params);
        this.walletConnector.connectionRestored.then((connected) => {
            if (!connected) {
                params[0](null);
            }
        });
        return unsubscribe;
    }
    disconnectWallet() {
        return this.walletConnector.disconnect();
    }
    createMessagePayload(message) {
        return (0, external_1.beginCell)().storeUint(0, 32).storeStringTail(message).endCell();
    }
    static createDependencies(params = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { connector, client, network = 'testnet', merchant } = params;
            const walletConnector = GameFiBase.isTonConnectUiInstance(connector)
                ? connector
                : GameFiBase.createConnectUiWorkaround(connector);
            let tonClient;
            if (client instanceof external_1.TonClient4) {
                tonClient = client;
            }
            else {
                let clientParams;
                if (client == null) {
                    const endpoint = yield (0, external_1.getHttpV4Endpoint)({
                        network
                    });
                    clientParams = { endpoint };
                }
                else {
                    clientParams = client;
                }
                tonClient = new external_1.TonClient4(clientParams);
            }
            const contentResolverParams = {};
            if (params.contentResolver != null) {
                const { ipfsGateway, urlProxy } = params.contentResolver;
                if (ipfsGateway != null) {
                    contentResolverParams.ipfsGateway = ipfsGateway;
                }
                if (urlProxy != null) {
                    if (typeof urlProxy === 'string') {
                        contentResolverParams.urlProxy = (url) => {
                            return urlProxy.replace(content_resolver_1.ProxyContentResolver.replaceable, url);
                        };
                    }
                    else {
                        contentResolverParams.urlProxy = urlProxy;
                    }
                }
            }
            const contentResolver = new content_resolver_1.ProxyContentResolver(contentResolverParams);
            const assetsSdk = external_1.AssetsSDK.create({
                api: {
                    openExtended: (contract) => {
                        return tonClient.openExtended(contract);
                    },
                    provider: (address, init) => tonClient.provider(address, init)
                },
                contentResolver: contentResolver,
                sender: new ton_connect_sender_1.TonConnectSender(walletConnector)
            });
            const dependencies = {
                walletConnector,
                assetsSdk
            };
            if (merchant != null) {
                dependencies.merchant = {
                    tonAddress: GameFiBase.addressToObject(merchant.tonAddress)
                };
                if (merchant.jettonAddress != null) {
                    dependencies.merchant.jettonAddress = GameFiBase.addressToObject(merchant.jettonAddress);
                }
            }
            return dependencies;
        });
    }
    static addressToObject(address) {
        if (typeof address === 'string') {
            return external_1.Address.parse(address);
        }
        return address;
    }
    static isTonConnectUiInstance(instance) {
        return typeof instance === 'object' && instance != null && 'openModal' in instance;
    }
    static createConnectUiWorkaround(params = {}) {
        const hide = (element) => {
            element.style.overflow = 'hidden';
            element.style.display = 'none';
            element.style.position = 'absolute';
            element.style.left = '-9999px';
        };
        const buttonRoot = document.createElement('div');
        buttonRoot.id = '__ton-connect-ui--button-root';
        hide(buttonRoot);
        document.body.appendChild(buttonRoot);
        const widgetRoot = document.createElement('div');
        widgetRoot.id = '__ton-connect-ui--widget-root';
        hide(widgetRoot);
        document.body.appendChild(widgetRoot);
        return new external_1.TonConnectUI(Object.assign(Object.assign({}, params), { buttonRootId: buttonRoot.id, widgetRootId: widgetRoot.id }));
    }
}
exports.GameFiBase = GameFiBase;
